/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 * Copyright (C) 2001 Vladimir Shebordaev <vshebordaev@mail.ru>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gemtek-pci.c,v 1.7 2002/01/14 10:21:34 pva Exp $
 * TEA5757: http://www.semiconductors.com/acrobat/datasheets/TEA5757_5759_3.pdf
 * lowlevel io functions for fmio utility -- Gemtek PCI driver
 */

#include <errno.h>
#include <stdio.h>

#include "access.h"
#include "tea5757.h"

#ifndef PCI_VENDOR_ID_GEMTEK
#define PCI_VENDOR_ID_GEMTEK 0x5046
#endif

#ifndef PCI_DEVICE_ID_GEMTEK_PR103
#define PCI_DEVICE_ID_GEMTEK_PR103 0x1001
#endif

#define GTP_WREN_ON	(1 << 2)
#define GTP_WREN_OFF	(0 << 2)
#define GTP_DATA_ON	(1 << 1)
#define GTP_DATA_OFF	(0 << 1)
#define GTP_CLCK_ON	(1 << 0)
#define GTP_CLCK_OFF	(0 << 0)

static unsigned long read_shift_register(void);
static void write_shift_register(unsigned long data);
static void send_zero(void);
static void send_one(void);

static unsigned long radioport = 0;
static struct tea5757_t card;

/*********************************************************************/

int
Grab_Port_gtp(unsigned long port) {
	if (radio_get_iopl() < 0)
		return -1;

	radioport = pci_bus_locate(
			PCI_VENDOR_ID_GEMTEK, PCI_DEVICE_ID_GEMTEK_PR103);

	if (radioport == 0) {
		errno = ENXIO;
		return -1;
	}

	/* Initialize the card */
	card.frequency = 0;
	card.search = TEA5757_SEARCH_END;
	card.sensitivity = TEA5757_S030;
	card.stereo = TEA5757_STEREO;
	card.write = write_shift_register;
	card.read = read_shift_register;

	return 0;
}

int
Release_Port_gtp(void) {
	return radio_release_iopl();
}

unsigned long
info_port_gtp(void) {
	return radioport;
}

/*
 * Set frequency and other stuff
 * Basically, this is just writing the 25-bit shift register
 */
void
Change_Frequency_gtp(unsigned long freq) {
	card.frequency = freq;
	card.search = TEA5757_SEARCH_END;
	tea5757_write_shift_register(&card);
	return;
}

unsigned long
search_up_gtp(unsigned long freq) {
	card.frequency = 0ul;
	card.search = TEA5757_SEARCH_UP;
	tea5757_write_shift_register(&card);
	/* Gemtek PCI is incapable to read the shift register */
	return 0ul;
}

unsigned long
search_down_gtp(unsigned long freq) {
	card.frequency = 0ul;
	card.search = TEA5757_SEARCH_DOWN;
	tea5757_write_shift_register(&card);
	return 0ul;
}

int
state_gtp(void) {
	int ret;

	OUTW(radioport, GTP_DATA_ON | GTP_WREN_OFF | GTP_CLCK_OFF);
	ret  = inw(radioport) & 8 ? 2 : 0; /* STEREO */
	OUTW(radioport, GTP_DATA_ON | GTP_WREN_OFF | GTP_CLCK_ON);
	ret += inw(radioport) & 8 ? 1 : 0; /* SIGNAL */
#if 0
	warnx("%s:%s", ret & 1 ? "noise" : "signal", ret & 2 ? "mono" : "stereo");
#endif

	OUTW(radioport, 0x10);

	return ret;
}

void
mono_gtp(void) {
	card.stereo = TEA5757_MONO;
}

static void
send_one(void) {
	OUTW(radioport, GTP_WREN_ON | GTP_DATA_ON | GTP_CLCK_OFF);
	OUTW(radioport, GTP_WREN_ON | GTP_DATA_ON | GTP_CLCK_ON);
	OUTW(radioport, GTP_WREN_ON | GTP_DATA_ON | GTP_CLCK_OFF);
}

static void
send_zero(void) {
	OUTW(radioport, GTP_WREN_ON | GTP_DATA_OFF | GTP_CLCK_OFF);
	OUTW(radioport, GTP_WREN_ON | GTP_DATA_OFF | GTP_CLCK_ON);
	OUTW(radioport, GTP_WREN_ON | GTP_DATA_OFF | GTP_CLCK_OFF);
}

void
mute_gtp(void) {
	OUTW(radioport, 0x1f);
}

void
unmute_gtp(void) {
	/* Set frequency, it is the only way to unmute the card */
	return;
}

static void
write_shift_register(unsigned long data) {
	int c = 25;

	OUTW(radioport, 0x06);

	while ( c-- )
		if (data & (1 << c))
			send_one();
		else
			send_zero();

	OUTW(radioport, 0x10);
}

static unsigned long
read_shift_register(void) {
	return 0ul;
}
