/*
 * Copyright (c) 2001, 2002 Vladimir Popov <jumbo@narod.ru>.
 * Copyright (c) 2000, 2001 Wilmer van der Gaast <lintux@lintux.cx>
 * Copyright (c) 1998 Kevin Boone <k.boone@mdx.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sf16fmr.c,v 1.21 2002/01/15 11:05:06 pva Exp $
 * lowlevel io functions for fmio utility -- sf16-fmr driver
 */

/*
 * Frequency is controlled by
 *   Toshiba's High Speed PLL for DTS TC9216P
 * Volume is controlled by
 *   Princeton Technology Corp.'s Electronic Volume Controller IC PT2254A.
 */

#include "access.h"
#include "pt2254a.h"
#include "tc921x.h"

#define SF16FMR_FREQ_DATA_ON	(1 << 0)
#define SF16FMR_FREQ_CLOCK_ON	(1 << 1)
#define SF16FMR_FREQ_PERIOD_ON	(1 << 2)

#define SF16FMR_FREQ_STEADY	\
		SF16FMR_FREQ_DATA_ON | \
		SF16FMR_FREQ_CLOCK_ON | \
		SF16FMR_FREQ_PERIOD_ON

#define SF16FMR_VOLU_STROBE_ON	(1 << 3)
#define SF16FMR_VOLU_STROBE_OFF	(0 << 3)
#define SF16FMR_VOLU_CLOCK_ON	(1 << 4)
#define SF16FMR_VOLU_CLOCK_OFF	(0 << 4)
#define SF16FMR_VOLU_DATA_ON	(1 << 5)
#define SF16FMR_VOLU_DATA_OFF	(0 << 5)

static struct tc921x_t card;
static int vol = 0;

static void set_vol(int);
static void send_vol_bit(int);
static int find_card_sf16fmr(int);


/******************************************************************/

int
Grab_Port_sf16fmr(unsigned long port) {
	card.port = port;

	card.period = SF16FMR_FREQ_PERIOD_ON;
	card.clock = SF16FMR_FREQ_CLOCK_ON;
	card.data = SF16FMR_FREQ_DATA_ON;

	return find_card_sf16fmr(port);
}

int
Release_Port_sf16fmr(void) {
	return radio_release_ioperms(card.port, 1);
}

unsigned long
info_port_sf16fmr(void) {
	return card.port;
}

void
Change_Frequency_sf16fmr(unsigned long frequency) {
	unsigned long data = 0ul;

	data  = tc921x_encode_freq(frequency);
	data |= TC921X_D0_REF_FREQ_10_KHZ;
	data |= TC921X_D0_PULSE_SWALLOW_FM_MODE;
	data |= TC921X_D0_OSC_7POINT2_MHZ;
	data |= TC921X_D0_OUT_CONTROL_OFF;
	tc921x_write_addr(&card, 0xD0, data);

	data  = TC921X_D2_GATE_TIME(0);
	data |= TC921X_D2_COUNTER_INPUT_LFC;
	data |= TC921X_D2_START_BIT;
	data |= TC921X_D2_IO_PORT_OUTPUT(4);
	tc921x_write_addr(&card, 0xD2, data);
}

unsigned long
get_freq_sf16fmr(void) {
	return tc921x_decode_freq(tc921x_read_addr(&card, 0xD1));
}

void
mute_sf16fmr(void) {
	vol = 0;
	set_vol(vol);
}

void
volumeup_sf16fmr(void) {
	if (vol < 15)
		vol++;

	set_vol(vol);
}

void
volumedn_sf16fmr(void) {
	if (vol > 0)
		vol--;

	set_vol(vol);
}

int
state_sf16fmr(void) {
#if 0
	u_int32_t d;
	int ret = 0;

	do {
		d = tc921x_read_addr(&card, 0xD1);
		warnx("0x%x", d);
	} while (d & TC921X_D1_BUSY);

	if (d & TC921X_D1_UNLOCK)
		ret = 2; /* mono */
	if ((d & TC921X_D1_COUNTER_DATA) == 0)
		ret += 1; /* no signal */

	return ret;
#else
	return 3;
#endif /* 0 */
}

static void
send_vol_bit(int i) {
	unsigned int data;

	data  = SF16FMR_FREQ_STEADY | SF16FMR_VOLU_STROBE_OFF;
	data |= i ? SF16FMR_VOLU_DATA_ON : SF16FMR_VOLU_DATA_OFF;

	OUTB(card.port, data | SF16FMR_VOLU_CLOCK_OFF);
	OUTB(card.port, data | SF16FMR_VOLU_CLOCK_ON);
}

static void
set_vol(int v) {
	unsigned long reg, vol;
	int i;

	vol = pt2254a_encode_volume(v, 15);
	reg = pt2254a_compose_register(vol, vol, USE_CHANNEL, USE_CHANNEL);

	OUTB(card.port, SF16FMR_FREQ_STEADY | SF16FMR_VOLU_STROBE_OFF);

	
	for (i = 0; i < PT2254A_REGISTER_LENGTH; i++)
		send_vol_bit(reg & (1 << i));

	/* Latch the data */
	OUTB(card.port, SF16FMR_FREQ_STEADY | SF16FMR_VOLU_STROBE_ON);
	OUTB(card.port, SF16FMR_FREQ_STEADY | SF16FMR_VOLU_STROBE_OFF);
}

static int
find_card_sf16fmr(int radioport) {
        unsigned long cur_freq = 0ul;
        unsigned long test_freq = 0ul;
        int err = -1;

        if (radio_get_ioperms(radioport, 1) < 0)
                return -1;

        cur_freq = get_freq_sf16fmr();

        Change_Frequency_sf16fmr(TEST_FREQ);
        test_freq = get_freq_sf16fmr();
        if (test_freq == TEST_FREQ) {
                Change_Frequency_sf16fmr(cur_freq);
                err = 0;
        }

        return err;
}

