/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sf64pcr.c,v 1.8 2002/01/14 10:21:39 pva Exp $
 * TEA5757: http://www.semiconductors.com/acrobat/datasheets/TEA5757_5759_3.pdf
 * lowlevel io functions for fmio utility -- sf64pcr driver
 */

#include <errno.h>

#include "access.h"
#include "tea5757.h"

#ifndef PCI_VENDOR_ID_FORTEMEDIA
#define PCI_VENDOR_ID_FORTEMEDIA	0x1319
#endif /* PCI_VENDOR_ID_FORTEMEDIA */

#ifndef PCI_DEVICE_ID_FORTEMEDIA_FM801
#define PCI_DEVICE_ID_FORTEMEDIA_FM801	0x0801
#endif /* PCI_VENDOR_ID_FORTEMEDIA_FM801 */

static void send_zero(void);
static void send_one(void);
static unsigned long read_shift_register(void);
static void write_shift_register(unsigned long);
static unsigned long read_shift_register(void);
static int find_card_sf64pcr(void);

static unsigned long radioport = 0;
static struct tea5757_t card;

/*********************************************************************/

int
Grab_Port_sf64pcr(unsigned long port) {
	if (radio_get_iopl() < 0)
		return -1;

	radioport = pci_bus_locate(
			PCI_VENDOR_ID_FORTEMEDIA, PCI_DEVICE_ID_FORTEMEDIA_FM801);
	if (radioport == 0) {
		errno = ENXIO;
		return -1;
	}
	radioport += 0x52;

	/* Initialize the card */
	card.frequency = 0;
	card.search = TEA5757_SEARCH_END;
	card.sensitivity = TEA5757_S030;
	card.stereo = TEA5757_STEREO;
	card.write = write_shift_register;
	card.read = read_shift_register;

	if (find_card_sf64pcr() < 0)
		return -1;

	return 0;
}

int
Release_Port_sf64pcr(void) {
	return radio_release_iopl();
}

unsigned long
info_port_sf64pcr(void) {
	return radioport;
}

void
mute_sf64pcr(void) {
	OUTW(radioport, 0xf800);
	usleep(6);
	OUTW(radioport, 0xf800);
}

void
unmute_sf64pcr(void) {
	OUTW(radioport, 0xf802);
	usleep(6);
	OUTW(radioport, 0xf802);
}

static void
send_zero(void) {
	OUTW(radioport, 0xf800);
	OUTW(radioport, 0xf801);
	OUTW(radioport, 0xf800);
}

static void
send_one(void) {
	OUTW(radioport, 0xf804);
	OUTW(radioport, 0xf805);
	OUTW(radioport, 0xf804);
}

/*
 * Set frequency and other stuff
 * Basically, this is just writing the 25-bit shift register
 */
void
set_frequency_sf64pcr(unsigned long freq) {
	card.frequency = freq;
	card.search = TEA5757_SEARCH_END;
	tea5757_write_shift_register(&card);
	return;
}

int
state_sf64pcr(void) {
	int ret = read_shift_register();
	return (ret >> 25) & 0x03;
}

void
mono_sf64pcr(void) {
	card.stereo = TEA5757_MONO;
}

static void
write_shift_register(unsigned long data) {
	int c = 25;

	OUTW(radioport, 0xf800);

	while (c--)
		if (data & (1 << c))
			send_one();
		else
			send_zero();

	OUTW(radioport, 0xf802);
}

unsigned long
read_shift_register(void) {
	unsigned long res = 0ul;
	int rb, ind = 0;

	OUTW(radioport, 0xfc02);
	usleep(4); 

	/* Read the register */
	rb = 23;
	while (rb--) {
		OUTW(radioport, 0xfc03);
		usleep(4);			

		OUTW(radioport, 0xfc02);
		usleep(4);

		res |= inw(radioport) & 0x04 ? 1 : 0;
		res <<= 1;
	}

	OUTW(radioport, 0xfc03);
	usleep(4);			

	rb = inw(radioport);
	ind = rb & 0x08 ? 1 : 0; /* Tuning */

	OUTW(radioport, 0xfc02);

	rb = inw(radioport);
	ind |= rb & 0x08 ? 2 : 0; /* Mono */
	res |= rb & 0x04 ? 1 : 0;

	return (res & (TEA5757_DATA | TEA5757_FREQ)) | (ind << 25);
}

unsigned long
get_frequency_sf64pcr(void) {
	return tea5757_decode_frequency(tea5757_read_shift_register(&card));
}

static int
find_card_sf64pcr(void) {
	unsigned long cur_freq, test_freq = 0ul;
	int err = -1;

	/* Save old value */
	cur_freq = get_frequency_sf64pcr();

	set_frequency_sf64pcr(TEST_FREQ);
	test_freq = get_frequency_sf64pcr();
	if (test_freq == TEST_FREQ)
		err = 0;

	set_frequency_sf64pcr(cur_freq);

	return err;
}
